#pragma NOIV                    // Do not generate interrupt vectors
//-----------------------------------------------------------------------------
//   File:      IRIS9002.c
//   Contents:  Hooks required to implement USB peripheral function.
//              
//   Copyright (c) 2001 Cypress Semiconductor All rights reserved
//-----------------------------------------------------------------------------
#include "fx2.h"
#include "fx2regs.h"
#include "fx2sdly.h"            // SYNCDELAY macro
#include "encoder.h"

void Write_Seg_LED (int);

#define ENCODER_N 5
encoder[ENCODER_N] = {0x30, 0x31, 0x32, 0x33, 0x34}; 																													//Encoder Address 

WORD counter[ENCODER_N]  = {0, 0, 0, 0, 0};																																		//Encoder default counter value
WORD maxvalue[ENCODER_N] = {20, 20, 20, 20, 20};																															//Encoder Max Value
WORD minvalue[ENCODER_N] = {-20, -20, -20, -20, -20};																													//Encoder Min Value
WORD econfig[ENCODER_N]  = {(INTE_ENABLE | LEDE_ENABLE | WRAP_DISABLE | DIRE_LEFT | IPUP_ENABLE | RMOD_X1 ),	//General config register
                            (INTE_ENABLE | LEDE_ENABLE | WRAP_DISABLE | DIRE_LEFT | IPUP_ENABLE | RMOD_X1 ),
                            (INTE_ENABLE | LEDE_ENABLE | WRAP_DISABLE | DIRE_LEFT | IPUP_ENABLE | RMOD_X1 ),
                            (INTE_ENABLE | LEDE_ENABLE | WRAP_DISABLE | DIRE_LEFT | IPUP_ENABLE | RMOD_X1 ),
                            (INTE_ENABLE | LEDE_ENABLE | WRAP_DISABLE | DIRE_LEFT | IPUP_ENABLE | RMOD_X1 ),
			  									 };
BYTE xdata ENCODER_DATA[4];
													 
WORD xdata Status;
BOOL ENCODER_RDY = FALSE;
// Digit algo for LED on prototype board.  Remove for production
BYTE xdata Digit[] = { 0xFF, 0x8C, 0xCF, 0xA1 }; // 7 SEG LED OFF/P/|/d
#define LED_ADDR 0x21

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;
extern BOOL Rwuen;
extern BOOL Selfpwr;


BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings

// EZUSB FX2 PORTC i/o...       port NA for 56-pin FX2
 sbit PC0 = IOC ^ 0;					
// sbit PC1 = IOC ^ 1;					
// sbit PC2 = IOC ^ 2;
// sbit PC3 = IOC ^ 3;
// sbit PC4 = IOC ^ 4;
// sbit PC5 = IOC ^ 5;
// sbit PC6 = IOC ^ 6;
// sbit PC7 = IOC ^ 7;



// EZUSB FX2 PORTE is not bit-addressable...

//-----------------------------------------------------------------------------
// Task Dispatcher hooks
//   The following hooks are called by the task dispatcher.
//-----------------------------------------------------------------------------

void TD_Init( void )
{ // Called once at startup
	REVCTL = 0x03;
  SYNCDELAY;
  CPUCS = 0x10;                 // CLKSPD[1:0]=10, for 48MHz operation
  SYNCDELAY;
  IFCONFIG = 0xCB;
  SYNCDELAY;
  EP1OUTCFG = (EP1OUTCFG & 0x7F);
	SYNCDELAY;
	EP1INCFG = (EP1INCFG & 0x7F);
	SYNCDELAY;
	EP2CFG = 0xE2;					// EP2 is VALID=1, 0=0, TYPE1=1, TYPE0=0, SIZE=0, 0=0, BUF1=1, BUF0=0 (DIR=IN, TYPE=BULK, SIZE=512, BUF=2x) 0xE2
	SYNCDELAY;
	EP4CFG = (EP4CFG & 0x7F);
	SYNCDELAY;
	EP8CFG = (EP8CFG & 0x7F);
	SYNCDELAY;
  EP6CFG = (EP6CFG & 0x7F);
  SYNCDELAY;

	FIFORESET = 0x80;
	SYNCDELAY;
	EP2FIFOCFG = 0x00;
	SYNCDELAY;
	FIFORESET = 0x02;
	SYNCDELAY;
	FIFORESET = 0x04;
	SYNCDELAY;
	FIFORESET = 0x06;
	SYNCDELAY;
	FIFORESET = 0x08;
	
	EP2FIFOCFG = 0x00;	
	SYNCDELAY;
	FIFORESET = 0x00;
	SYNCDELAY;

  // We want to get SOF interrupts
	USBIE |= bmSOF + bmURES;
	SYNCDELAY;
	PORTCCFG &= 0x00;
	SYNCDELAY;
	OEC = 0x00;
	SYNCDELAY;
  Rwuen = TRUE;                 																														// Enable remote-wakeup
	SYNCDELAY;
  I2CTL &= ~bm400KHZ;
	ENCODER_RDY = FALSE;																																	  	
}

void TD_Poll( void )
{ // Called repeatedly while the device is idle
	
	int i;
	WORD ENCODER_ZERO = 0x00;
	//BYTE xdata ENCODER_DATA[8];
	
	//EZUSB_Delay(4);
	if(!(EP1INCS & bmEPBUSY))
	{	
		if (ENCODER_RDY) 
			{
				if (!(PC0))
					{
					
					for(i=0; i <= ENCODER_N; i++)
						{
						Encoder_Read(encoder[i], ENCODER_STATUS);
						switch(Status)
							{
							case E_PUSH:
								Write_Seg_LED(1);		//0x01								//displays a P on the seven segment led on dev board
								break;
							case E_INCREMENT:		//0x0A
								Write_Seg_LED(2);										//displays a | on the seven segment led on dev board
								Encoder_LED(encoder[i], INC);
								Encoder_Read(encoder[i], DATA_COUNTER_BYTE1);
								break;
							case E_DECREMENT:		//0x14
								Write_Seg_LED(3);										//displays a d on the seven segment led on dev board
								Encoder_LED(encoder[i], DEC);
								Encoder_Read(encoder[i], DATA_COUNTER_BYTE1);
								break;
							
							}	
							ENCODER_DATA[1] = Status & 0xFF;
							EP2FIFOBUF[i+1] = ENCODER_DATA[1];
						}	
					}	
				SYNCDELAY;
				SYNCDELAY;
				EP2BCH=0x00;
				SYNCDELAY;
				EP2BCL=0x10;
				SYNCDELAY;
		
			}
	}
}

BOOL TD_Suspend( void )          
{ // Called before the device goes into suspend mode
   return( FALSE );		//originally True
}

BOOL TD_Resume( void )          
{ // Called after the device resumes
   return( TRUE );
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------
BOOL DR_GetDescriptor( void )
{
	return( TRUE );
}

BOOL DR_SetConfiguration( void )   
{ // Called when a Set Configuration command is received 
  Configuration = SETUPDAT[ 2 ];
  return( TRUE );        // Handled by user code
}

BOOL DR_GetConfiguration( void )   
{ // Called when a Get Configuration command is received
   EP0BUF[ 0 ] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);          // Handled by user code
}

BOOL DR_SetInterface( void )       
{ // Called when a Set Interface command is received
   return( TRUE );        // Handled by user code
}

BOOL DR_GetInterface( void )       
{ // Called when a Set Interface command is received
   EP0BUF[ 0 ] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return( TRUE );        // Handled by user code
}

BOOL DR_GetStatus( void )
{
   return( TRUE );
}

BOOL DR_ClearFeature( void )
{
   return( TRUE );
}

BOOL DR_SetFeature( void )
{
   return( TRUE );
}


//----------------------------------------------------------------------------
// Conversion of SETUPDAT from EZUSB Vendor Request Tool Bar.
//----------------------------------------------------------------------------

		// SETUPDAT[0] = Direction of Transfer (0x40 = IN, 0xC0 = OUT)
		// SETUPDAT[1] = Req
		// SETUPDAT[2] = LSB of Value
		// SETUPDAT[3] = MSB of Value
		// SETUPDAT[4] = LSB of INDEX
		// SETUPDAT[5] = MSB of INDEX
		// SETUPDAT[6] = LSB of LENGTH (Set to 0 unless doing an A2 or FD request) 
		// SETUPDAT[7] = MSB of LENGTH (Set to 0 unless doing an A2 or FD request) 

#define VR_DOWNLOAD 	0x40
#define VR_UPLOAD 		0xC0

#define VR_EEPROM			0xA2
#define SW_RESET			0xB2
#define PROGRAM_FW		0xB5
#define EEPROM				0xB8
#define I2C_CTL				0xFD
#define INIT_ENCODER	0xFE

#define SERIAL_ADDR		0x50 // Base Address for serial EEPROM
#define EP0BUFF_SIZE	0x40 // Max buffer size for EP0

//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------
BYTE			DB_Addr;					//TPM Dual Byte Address stat
BYTE			I2C_Addr;					//TPM I2C address

//-----------------------------------------------------------------------------
// Prototypes
//-----------------------------------------------------------------------------
void EEPROMWrite(WORD addr, BYTE length, BYTE xdata *buf); //TPM EEPROM Write
void EEPROMRead(WORD addr, BYTE length, BYTE xdata *buf);  //TPM EEPROM Read

BOOL DR_VendorCmnd(void)
{
  WORD		addr, len, bc;
  WORD		index;
	
  switch(SETUPDAT[1])
	{ //TPM handle new commands
		case SW_RESET:
					*EP0BUF = 7;
					EP0BCH = 0;
					EP0BCL = 1; // Arm endpoint with # bytes to transfer
					EP0CS |= bmHSNAK; // Acknowledge handshake phase of device request
					EZUSB_Delay(1000);
					EZUSB_Discon(TRUE);		// renumerate until setup received
		break;
		
		case INIT_ENCODER:
			initialize_encoder();
			
		break;
		
		case VR_EEPROM:
			
		case EEPROM:
			// No break so when EEPROM is called it runs the I2C_CTL code     
	  case I2C_CTL:			//Controls IRIS I2C Controls  
	  
            /*
                To Change the I2C Controls using a Vendor Request enter the following values:
                				Vendor Req.														EEPROM Req.
								REQ         = 	0xFD				 	  							0xA2
                VALUE       = 	Device Address 					  		Address to begin Write
                INDEX       = 	Mux Address 									Mux Address 
                LENGTH      = 	Number of bytes to transfer 	Number of bytes to transfer
                DIR         = 	OUT for Write									OUT for Write, IN for Read
                HEX BYTES   = 	Data Out to I2C 							I/O Data
	        */
            
			addr = SETUPDAT[2];		// Get device address, mux address, and length
			addr |= SETUPDAT[3] << 8;
			index = SETUPDAT[4];		
			index |= SETUPDAT[5] <<8;	
			len = SETUPDAT[6];
			len |= SETUPDAT[7] << 8;
			
			if(SETUPDAT[0] == VR_UPLOAD)	

			{
				while(len)					// Move requested data through EP0IN 
				{							// one packet at a time.
			 		while(EP0CS & bmEPBUSY);

					if(len < EP0BUFF_SIZE)
						bc = len;
					else
						bc = EP0BUFF_SIZE;
					
					if (SETUPDAT[1] == I2C_CTL)								//Normmally we will only write to the I2C controls
					{
										
						EZUSB_ReadI2C(addr,len,(WORD)EP0BUF);
					}
					else																			//Read the EEPROMs
					{					
						if(index == 0x10)
							I2C_Addr = 0x50;
						else
							I2C_Addr = 0x51;
		
						EEPROMRead(addr,(WORD)bc,(WORD)EP0BUF);
					}
					EP0BCH = 0;
					EP0BCL = (BYTE)bc; // Arm endpoint with # bytes to transfer
					addr += bc;
					len -= bc;
				}
			}
			// Is this a download command ?
			else if(SETUPDAT[0] == VR_DOWNLOAD)
			{
				while(len)					// Move new data through EP0OUT 
				{							// one packet at a time.
					// Arm endpoint - do it here to clear (after sud avail)
					EP0BCH = 0;
					EP0BCL = 0; // Clear bytecount to allow new data in; also stops NAKing

					while(EP0CS & bmEPBUSY);

					bc = EP0BCL; // Get the new bytecount
					if (SETUPDAT[1] == I2C_CTL)  //Program I2C Controls
					{
						EZUSB_WriteI2C(addr,len,(WORD)EP0BUF);
					}	
					else if ((SETUPDAT[1] ==  EEPROM) || (SETUPDAT[1] == VR_EEPROM))   //Program EEPROMs
					{	
						if(index == 0x10)				
						{
							I2C_Addr = 0x50;
						}
						else																	//Generally speaking the API call will write to the firmware
						{
							I2C_Addr = 0x51;
						}
						EEPROMWrite(addr,bc,(WORD)EP0BUF);						
					}
						
					addr += bc;
					len -= bc;
				}
								
			}	    
	  break;
		default:
				//  Force Renumeration
  		EZUSB_Discon(TRUE);
			break;
    
	}
  	return(FALSE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------

// Setup Data Available Interrupt Handler
void ISR_Sudav( void ) interrupt 0
{
   	GotSUD = TRUE;         // Set flag
   	EZUSB_IRQ_CLEAR( );
   	USBIRQ = bmSUDAV;      // Clear SUDAV IRQ

}

// Setup Token Interrupt Handler
void ISR_Sutok( void ) interrupt 0
{
   	EZUSB_IRQ_CLEAR( );
   	USBIRQ = bmSUTOK;      // Clear SUTOK IRQ
}

void ISR_Sof( void ) interrupt 0
{
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSOF;        // Clear SOF IRQ
}

void ISR_Ures( void ) interrupt 0
{
   if ( EZUSB_HIGHSPEED( ) )
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
   }
   
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmURES;       // Clear URES IRQ
}									 

void ISR_Susp( void ) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmSUSP;
}

void ISR_Highspeed( void ) interrupt 0
{
   if ( EZUSB_HIGHSPEED( ) )
   {
      pConfigDscr = pHighSpeedConfigDscr;
      pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
      pConfigDscr = pFullSpeedConfigDscr;
      pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR( );
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack( void ) interrupt 0
{
}
void ISR_Stub( void ) interrupt 0
{
}
void ISR_Ep0in( void ) interrupt 0
{
}
void ISR_Ep0out( void ) interrupt 0
{
}
void ISR_Ep1in( void ) interrupt 0
{
}
void ISR_Ep1out( void ) interrupt 0
{
}
void ISR_Ep2inout( void ) interrupt 0
{
}
void ISR_Ep4inout( void ) interrupt 0
{
}
void ISR_Ep6inout( void ) interrupt 0
{
}
void ISR_Ep8inout( void ) interrupt 0
{
}
void ISR_Ibn( void ) interrupt 0
{
}
void ISR_Ep0pingnak( void ) interrupt 0
{
}
void ISR_Ep1pingnak( void ) interrupt 0
{
}
void ISR_Ep2pingnak( void ) interrupt 0
{
}
void ISR_Ep4pingnak( void ) interrupt 0
{
}
void ISR_Ep6pingnak( void ) interrupt 0
{
}
void ISR_Ep8pingnak( void ) interrupt 0
{
}
void ISR_Errorlimit( void ) interrupt 0
{
}
void ISR_Ep2piderror( void ) interrupt 0
{
}
void ISR_Ep4piderror( void ) interrupt 0
{
}
void ISR_Ep6piderror( void ) interrupt 0
{
}
void ISR_Ep8piderror( void ) interrupt 0
{
}
void ISR_Ep2pflag( void ) interrupt 0
{
}
void ISR_Ep4pflag( void ) interrupt 0
{
}
void ISR_Ep6pflag( void ) interrupt 0
{
}
void ISR_Ep8pflag( void ) interrupt 0
{
}
void ISR_Ep2eflag( void ) interrupt 0
{
}
void ISR_Ep4eflag( void ) interrupt 0
{
}
void ISR_Ep6eflag( void ) interrupt 0
{
}
void ISR_Ep8eflag( void ) interrupt 0
{
}
void ISR_Ep2fflag( void ) interrupt 0
{
}
void ISR_Ep4fflag( void ) interrupt 0
{
}
void ISR_Ep6fflag( void ) interrupt 0
{
}
void ISR_Ep8fflag( void ) interrupt 0
{
}
void ISR_GpifComplete( void ) interrupt 0
{
}
void ISR_GpifWaveform( void ) interrupt 0
{
}
void EEPROMWriteByte(WORD addr, BYTE value)
{
	BYTE		i = 0;
	BYTE xdata 	ee_str[3];

	ee_str[i++] = MSB(addr);
	ee_str[i++] = LSB(addr);
	ee_str[i++] = value;

	EZUSB_WriteI2C(I2C_Addr, i, ee_str);
  EZUSB_WaitForEEPROMWrite(I2C_Addr);
}

void EEPROMWrite(WORD addr, BYTE length, BYTE xdata *buf)
{
	BYTE	i;
	for(i=0;i<length;++i)
		EEPROMWriteByte(addr++,buf[i]);
}

void EEPROMRead(WORD addr, BYTE length, BYTE xdata *buf)
{
	BYTE		i = 0;
	BYTE		j = 0;
	BYTE xdata 	ee_str[2];

	ee_str[i++] = MSB(addr);
	ee_str[i++] = LSB(addr);

	EZUSB_WriteI2C(I2C_Addr, i, ee_str);		//Moves the pointer to the correct eeprom memory location
	EZUSB_ReadI2C(I2C_Addr, length, buf);
}
void initialize_encoder()
{
		int i;
			
		for (i=0; i <= ENCODER_N; i++)
		{		
			Encoder_Write(encoder[i], GENERAL_CONFIG, 2, econfig[i]); 	  //0x00	0x0B
			Encoder_Write(encoder[i], LED_A_PWM, 2, LED_OFF);							//0x0E	0x00
			Encoder_Write(encoder[i], LED_B_PWM, 2, LED_OFF);							//0x0F	0x00
			Encoder_Write(encoder[i], DATA_COUNTER_BYTE4, 5, counter[i]);	//0x05	0x00
			Encoder_Write(encoder[i], DATA_MAX_BYTE4, 5, maxvalue[i]);		//0x09	0x14
			Encoder_Write(encoder[i], DATA_MIN_BYTE4, 5, minvalue[i]);		//0x0D	0xEC
			Encoder_Read(encoder[i], ENCODER_STATUS);															 
		}
		for (i=0;i<16;i++)
		{
			EP2FIFOBUF[i] = 0;
		}
		
		EZUSB_WriteI2C(LED_ADDR, 1, &Digit[0]);	
		ENCODER_RDY = TRUE;
}
void Encoder_Write(WORD addr, WORD reg, WORD len, WORD Data)
{
			int i;
			//BYTE xdata ENCODER_DATA[4];
			
			for(i=0;i<4;i++)
			{
				ENCODER_DATA[i]=0x00;
			}
			ENCODER_DATA[0] = reg;
			if (Data > 0x7F){
				for(i=1 ;i<3 ;i++)
				{
					ENCODER_DATA[i] = 0xFF;
				}
				ENCODER_DATA[3] = Data & 0xFF;
			}
			else
			{
				ENCODER_DATA[len - 1] = Data;
			}
			EZUSB_WriteI2C(addr , len, &ENCODER_DATA);					//write the data
			EZUSB_Delay(5);
}
void Encoder_Read(WORD addr, WORD reg)
{
			int i;
			//BYTE xdata ENCODER_DATA[4];
	
			Status = 0x00;
			for(i=0;i<4;i++)
			{
				ENCODER_DATA[i]=0x00;
			}
			ENCODER_DATA[0] = reg;
			EZUSB_WriteI2C(addr, 1, ENCODER_DATA);						//set the correct register to read from
			EZUSB_ReadI2C(addr, 1, &EP0BUF);								//read register
			Status = EP0BUF[0];
			EZUSB_Delay(5);
}
void Write_Seg_LED(int num)
{
			EZUSB_WriteI2C(LED_ADDR, 1, &Digit[num]);
			EZUSB_Delay(50);
			EZUSB_WriteI2C(LED_ADDR, 1, &Digit[0]);
			EZUSB_Delay(5);
}
void Encoder_LED(WORD addr, BOOL direction)
{
		
		if(direction)
			{
				Encoder_Write(addr, LED_B_PWM, 2, LED_ON);
				EZUSB_Delay(50);
				Encoder_Write(addr, LED_B_PWM, 2, LED_OFF);
			}
			else
			{
				Encoder_Write(addr, LED_A_PWM, 0x02, LED_ON);
				EZUSB_Delay(50);
				Encoder_Write(addr, LED_A_PWM, 0x02, LED_OFF);
			}
			EZUSB_Delay(5);
}